package MonteCarlo;

/* The KnockAbout GameBoard
 * ========================
 * The Knockabout GameBoard is simply of a double-array of Hexes.
 * Due to the shape of the board (see game pics on website), not every element
 * in the array represents a valid board position. We can use the isValidCoord
 * method to verify that an (x,y) position on the gameboard is valid.
 *

  uses the coordinate system as depicted on http://www.gamerz.net/pbmserv/hexxagon.html
  I say we scrap the use of letters and start at 0 on each side. With x's on the left and
  y's on the topright as so (o's represent the gutter):

                0 1 2 3 4 5 6
               / / / / / / / 7
          0-  o o o o o o o / 8     y
         1-  o . . . . . . o / 9
 x      2-  o . . . . . . . o / 10
       3-  o . . . . . . . . o / 11
      4-  o . . . . . . . . . o / 12
     5-  o . . . . . . . . . . o /
    6-  o . . . . . . . . . . . o
     7-  o . . . . . . . . z . o    <--- that 'z' is at loc (x,y) = (7,10)
      8-  o . . . . . . . . . o
       9-  o . . . . . . . . o
       10-  o . . . . . . . o
        11-  o . . . . . . o
         12-  o o o o o o o

- mutable data type
- the starting position is as follows:
  initial face-up values: 
    all four-sided dice: 1
	 all six-sided dice: 2
	 all eight-sided dice: 3
  positions:
	  (3,4) black d4
	  (3,5) yellow d4
	  (4,3) black d6
	  (4,7) yellow d6
	  (5,2) black d8
	  (5,5) black d4
	  (5,6) yellow d4
	  (5,9) yellow d8
	  (6,4) black d6)
	  (6,8) yellow d6
	  (7,3) black d8
	  (7,6) black d4
	  (7,7) yellow d4
	  (7,10) yellow d8
	  (8,5) black d6
	  (8,9) yellow d6
	  (9,7) black d4
	  (9,8) yellow d4

 */

class Board{
    
    //Variables
    private static final int BOARDSIZE = 13; //We use a 13x13 Hex Grid
    private Hex[][] board; //Game Board consists of a 2d-array of hexes.
    
    //New Instance of HexBoard
    public Board(){
        board = new Hex[BOARDSIZE][BOARDSIZE];
        
       //Initialize the Board to Empty
       for (int y = 0;y < BOARDSIZE;y++)
           for (int x = 0;x < BOARDSIZE;x++)
               if (isValidCoord(x,y)) board[x][y] = new Hex();

       //Place Initial Pieces on Board
        
        //Initialize Black Dies and places them on the game board
        board[5][2].putDie(new Die(Player.BLACK,8,3));
        board[7][3].putDie(new Die(Player.BLACK,8,3));
        board[4][3].putDie(new Die(Player.BLACK,6,2));
        board[6][4].putDie(new Die(Player.BLACK,6,2));
        board[8][5].putDie(new Die(Player.BLACK,6,2));
        board[3][4].putDie(new Die(Player.BLACK,4,1));
        board[5][5].putDie(new Die(Player.BLACK,4,1));
        board[7][6].putDie(new Die(Player.BLACK,4,1));
        board[9][7].putDie(new Die(Player.BLACK,4,1));
       
       //Initialize Yellow Dies and places them on the game board 
        board[5][9].putDie(new Die(Player.YELLOW,8,3));
        board[7][10].putDie(new Die(Player.YELLOW,8,3));
        board[4][7].putDie(new Die(Player.YELLOW,6,2));
        board[6][8].putDie(new Die(Player.YELLOW,6,2));
        board[8][9].putDie(new Die(Player.YELLOW,6,2));
        board[3][5].putDie(new Die(Player.YELLOW,4,1));
        board[5][6].putDie(new Die(Player.YELLOW,4,1));
        board[7][7].putDie(new Die(Player.YELLOW,4,1));
        board[9][8].putDie(new Die(Player.YELLOW,4,1));
       
        
    }
    
    //Is position (x,y) located in the Gutter area?
    public boolean inGutter(int x, int y)
	{
		if (   (x == 0 && y == 0) || (x == 1 && y == 0) || (x == 2 && y == 0) || (x == 3 && y == 0)
		    || (x == 4 && y == 0) || (x == 5 && y == 0) || (x == 6 && y == 0) || (x == 7 && y == 1)
			 || (x == 8 && y == 2) || (x == 9 && y == 3) || (x == 10 && y == 4) || (x == 11 && y == 5)
			 || (x == 12 && y == 6) || (x == 12 && y == 7) || (x == 12 && y == 8) || (x == 12 && y == 9)
			 || (x == 12 && y == 10) || (x == 12 && y == 11) || (x == 12 && y == 12) || (x == 11 && y == 12)
			 || (x == 10 && y == 12) || (x == 9 && y == 12) || (x == 8 && y == 12) || (x == 7 && y == 12)
			 || (x == 6 && y == 12) || (x == 5 && y == 11) || (x == 4 && y == 10) || (x == 3 && y == 9)
			 || (x == 2 && y == 8) || (x == 1 && y == 7) || (x == 0 && y == 6) || (x == 0 && y == 5)
			 || (x == 0 && y == 4) || (x == 0 && y == 3) || (x == 0 && y == 2) || (x == 0 && y == 1) )
			return true;
		else
			return false;
	}
    
     
    // Is this (x,y) coordinate located on the Hex-Grid
     public boolean isValidCoord(int x, int y){
      
      final int sideLength = 13;
      
      int median= (sideLength/2);
      if (x < 0 || x >= sideLength || y < 0 || y >= sideLength)
         return false;     
      else if (x > median && x > y && (x-y) > median)         // bottom left region
         return false;      
      else if (y > median && y > x && (y-x) > median)         // top right region
         return false;      
      return true;
        }
  
     //Moves a die from one hex at (x1,y1) to the position (x2,y2)
     private void move(int x1,int y1,int x2,int y2){
         Die die = board[x1][y1].getDie();
         board[x2][y2].putDie(die);
         board[x1][y1].putDie(null);
     }
     
     //Returns true if the given move by black or yellow player is a legal move,
     //otherwise false
     public boolean isValidMove(Move move,int playercolor){         
         int x = move.getX();
         int y = move.getY();
         int dir = move.getDir();
         Hex hex = board[x][y];
         
         if (inGutter(x,y))
         {
             if (isValidGutterMove(move) && hex.isOccupied() &&
         (hex.getDie().getColor() == playercolor)) return(true);else
             return(false);
         }
         
         else if (isValidCoord(x,y)){
            if ((dir == Move.DIR_NE || dir == Move.DIR_NW || dir == Move.DIR_E
            || dir == Move.DIR_W || dir == Move.DIR_SE || dir == Move.DIR_SW)
            && hex.isOccupied() && (hex.getDie().getColor() == playercolor))
                return(true);else return(false);
         }
         
         else return(false);
            
     }
    
    //Returns true if a move within the gutter area is valid. Otherwise false
    private boolean isValidGutterMove(Move m)
    {
        return(isValidGutterMove(m.getX(),m.getY(),m.getDir()));
    }
    
   //Returns true if a move within the gutter area is valid. Otherwise false
    private boolean isValidGutterMove(int x, int y, int dir)
	{
		if (x == 0 && y == 0)
			if (dir == Move.DIR_SW || dir == Move.DIR_E)
				return true;

		if (x >= 1 && x <= 5 && y == 0)
			if (dir == Move.DIR_NE || dir == Move.DIR_SW)
				return true;

		if (x == 6 && y == 0)
			if (dir == Move.DIR_NE || dir == Move.DIR_SE)
				return true;

		if (   (x == 7 && y == 1) || (x == 8 && y == 2) || (x == 9 && y == 3)
		    || (x == 10 && y == 4) || (x == 11 && y == 5))
			 if (dir == Move.DIR_NW || dir == Move.DIR_SE)
			 	return true;

		if (x == 12 && y == 6)
			if (dir == Move.DIR_NW || dir == Move.DIR_E)
				return true;

		if (x == 12 && y >= 7 && y <= 11)
			if (dir == Move.DIR_W || dir == Move.DIR_E)
				return true;

		if (x == 12 && y == 12)
			if (dir == Move.DIR_W || dir == Move.DIR_NE)
				return true;

		if (x <= 11 && x >= 7 && y == 12)
			if (dir == Move.DIR_SW || dir == Move.DIR_NE)
				return true;

		if (x == 6 && y == 12)
			if (dir == Move.DIR_SW || dir == Move.DIR_NW)
				return true;

		if (   (x == 5 && y == 11) || (x == 4 && y == 10) || (x == 3 && y == 9)
		    || (x == 2 && y == 8) || (x == 1 && y == 7))
			 if (dir == Move.DIR_NW || dir == Move.DIR_SE)
			 	return true;

		if (x == 0 && y == 6)
			if (dir == Move.DIR_W || dir == Move.DIR_SE)
				return true;

		if (x == 0 && y <= 5 && y >= 1)
			if (dir == Move.DIR_W || dir == Move.DIR_E)
				return true;

		return false;
	}
   
    //Returns the Hexagon located at Board coordinate (x,y). If there is no
    //Hexagon (i.e. it is not a legal board position), it will return null.
    public Hex getHex(int x,int y){
        return(board[x][y]);
    }
    
    
 /* Private Helper Functions: nextX,nextY,nextDir. Used for manuevering around the board
 *======================================================================================
 */
    
    private int nextX(int x, int dir)
	{
		switch(dir)
		{
			case Move.DIR_NW: return (x-1);
			case Move.DIR_NE: return (x-1);
			case Move.DIR_W: return (x);
			case Move.DIR_E: return (x);
			case Move.DIR_SW: return (x+1);
			case Move.DIR_SE: return (x+1);
			default: return -10000;
		}
	}

	private int nextY(int y, int dir)
	{
		switch(dir)
		{
			case Move.DIR_NW: return (y-1);
			case Move.DIR_NE: return (y);
			case Move.DIR_W: return (y-1);
			case Move.DIR_E: return (y+1);
			case Move.DIR_SW: return (y);
			case Move.DIR_SE: return (y+1);
			default: return -10000;
		}
	}

        private int nextDir(int x, int y, int dir)
	{ //Only used when moving around the gutter area
		int newdir = dir;

		if      (x == 1 && y == 0 && dir == Move.DIR_NE)
			newdir = Move.DIR_E;
		else if (x == 0 && y == 1 && dir == Move.DIR_W)
			newdir = Move.DIR_SW;
		else if (x == 5 && y == 0 && dir == Move.DIR_SW)
			newdir = Move.DIR_SE;
		else if (x == 7 && y == 1 && dir == Move.DIR_NW)
			newdir= Move.DIR_NE;
		else if (x == 11 && y == 5 && dir == Move.DIR_SE)
			newdir = Move.DIR_E;
		else if (x == 12 && y == 7 && dir == Move.DIR_W)
			newdir= Move.DIR_NW;
		else if (x == 12 && y == 11 && dir == Move.DIR_E)
			newdir= Move.DIR_NE;
		else if (x == 11 && y == 12 && dir == Move.DIR_SW)
			newdir= Move.DIR_W;
		else if (x == 7 && y == 12 && dir == Move.DIR_NE)
			newdir= Move.DIR_NW;
		else if (x == 5 && y == 11 && dir == Move.DIR_SE)
			newdir= Move.DIR_SW;
		else if (x == 1 && y == 7 && dir == Move.DIR_NW)
			newdir= Move.DIR_W;
		else if (x == 0 && y == 5 && dir == Move.DIR_E)
			newdir= Move.DIR_SE;

		return newdir;
	}
 
   
   //Performs a deep clone of the board        
   public Board deepclone(){
       Board newboard = new Board();
       
       for (int i = 0;i < BOARDSIZE;i++)
           for (int j = 0;j < BOARDSIZE;j++)
               if (isValidCoord(i,j)){
                   newboard.board[i][j] = board[i][j].deepclone();
                   
               }
      return(newboard);
   }

    
   //Applies the supplied game move by player color 'playercolor' on the
   //game board. Should a collision occur, the newval parameter is the value 
   //of the die that gets rerolled.
 
    public int applyMove(Move move, int playerColor, int newval)
	{
                int x,y,dir;
                boolean gutterMove = false;
                
                // check out of bounds
		if (!isValidMove(move,playerColor)){
                    System.out.println("Invalid move: " + move);
                    return -1;
                }
                       
                x = move.getX();
                y = move.getY();
                dir = move.getDir();
                
                if (inGutter(x,y)) gutterMove = true;
                
		// we've done all the validation we need, now apply the move
		int movementPoints= getHex(x,y).getDie().curVal();
		int curX= x;
		int curY= y;
		int nextX= x;
		int nextY= y;
		int retval= 0;
		boolean collision= false;

		int curDie_x= x;
		int curDie_y= y;
		Die curDie= getHex(x,y).getDie();

		while(movementPoints > 0){
                    
                    nextX = this.nextX(curX,dir);
                    nextY = this.nextY(curY,dir);
                    
                    if (gutterMove) dir = this.nextDir(curX,curY,dir);
                    
                    if (!(isValidCoord(nextX,nextY))) break;
                    
                    if (getHex(nextX,nextY).isOccupied())
                    {
                        collision = true;
                        board[curDie_x][curDie_y].putDie(null);
                        board[curX][curY].putDie(curDie);
                        curDie_x = nextX;
                        curDie_y = nextY;
                        curDie = getHex(nextX,nextY).getDie();
                    }
                    else
                    {
                        movementPoints = movementPoints - 1;
                    }
                    
                    curX = nextX;
                    curY = nextY;
                    
                } //end of while loop
                
                board[curDie_x][curDie_y].putDie(null);
                board[curX][curY].putDie(curDie);
                
                if (collision) curDie.setVal(newval);
                return(1);
    }
} //End of Board Class
