package MonteCarlo;

import java.net.*;
import java.io.*;
import java.util.*;

/* The Knockabout Game Client
 * ==========================
 * This is a Java implementation of the Knockabout Game client.
 * Usage: <GameClient> <ServerName> <Port>
 */

public class GameClient{

    public static void main(String[] args) {
        
        if (args.length != 2) usage();
        
        int x,y; //Screen Coordinates
        Board board = new Board();
        String szServerMachine = args[0];
        int iPortNum = Integer.parseInt(args[1]);
        boolean isGameOver = false;
        Move move;
        
        System.out.println("Connecting to "+szServerMachine + " on port " + iPortNum);
        
        Socket clientSocket = null;
        AIPlayer player = new AIPlayer(); //Creates a New Player
        String msg, firstToken,secondToken;
        String TokenX,TokenY,TokenZ,CurValToken;
        String szWinner;
        
        int new_die_val;
        StringTokenizer msgTokenizer,winTokenizer;
        int iCurrentTurn = Player.YELLOW,iMyColor = Player.NOCOLOR,iWinnerColor = Player.NOCOLOR;
        
        try{
            clientSocket = new Socket(szServerMachine, iPortNum);
            PrintWriter out = new PrintWriter(clientSocket.getOutputStream(), true);
            BufferedReader in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
            
            out.println("START " + player.getMyName()); //Sends Player Name to Game Server
            msg = new String("");
            
            while (!msg.startsWith("GAME STARTED"))
            {
            msg = in.readLine(); //Waits for server response.
            };
            
            System.out.println("Server: "+msg);
            
            //Retrieves Current Player's Assigned Color
            if (msg.startsWith("GAME STARTED; YOU ARE YELLOW")) iMyColor = Player.YELLOW;
            else iMyColor = Player.BLACK; //Sets your player's color
            
            System.out.print("Game Started. My Color is ");
            if (iMyColor == Player.YELLOW) System.out.println("Yellow");
            else if (iMyColor == Player.BLACK) System.out.println("BLACK");
            else System.out.println("[ERROR. NO COLOR PARSED]");
            
            //Sets Player Color Returned By Server
            player.setMyColor(iMyColor);
            
            //If player is Starting Player, send your first move
            if (iMyColor == Player.YELLOW){
                move = player.chooseMove(board);
                msg = new String("YELLOW ");
                msg = msg.concat(move.toString());
                out.println(msg);
                System.out.println("Sent Move to Server: "+msg);
                iCurrentTurn = Player.BLACK;
            }
            
            //Use StringTokenizer to Scan String and Determine Color
            //Initialize Other Game Variables According to msg
            
            do
            {
                msg = in.readLine();
                System.out.println("Server: "+msg);
                msgTokenizer = new StringTokenizer(msg, " ");
                firstToken = msgTokenizer.nextToken();
                //Checks for Game Ended
                 if (firstToken.equals("GAME")){
                    secondToken = msgTokenizer.nextToken();
                    if (secondToken.startsWith("ENDED"))
                    {
                    winTokenizer = new StringTokenizer(msg,";");                        
                    isGameOver = true;
                    szWinner = winTokenizer.nextToken();
                    szWinner = winTokenizer.nextToken();
                    szWinner = winTokenizer.nextToken();
                    if (szWinner.startsWith(" WINNER IS BLACK")) iWinnerColor = Player.BLACK;
                    else if (szWinner.startsWith(" WINNER IS YELLOW")) iWinnerColor = Player.YELLOW;
                    else if (szWinner.startsWith(" DRAW")) iWinnerColor = Player.DRAW;
                    else iWinnerColor = Player.NOCOLOR;
                    if (iWinnerColor == Player.BLACK) System.out.println("Game is Over. Winner is BLACK");
                    else if (iWinnerColor == Player.YELLOW) System.out.println("Game is Over. Winner is YELLOW");
                    else if (iWinnerColor == Player.DRAW) System.out.println("Game is Over. Outcome is a Draw");
                    else System.out.println("Game is over. Winner indeterminant. This shouldn't happen");
                    }
                }
                else if (firstToken.equals("BLACK") || firstToken.equals("YELLOW")){
                    //Parse Next 2 Strings
                    TokenX = msgTokenizer.nextToken();
                    if (!TokenX.equals("TIMEOUT") && !TokenX.equals("INVALID")){ 
                        //A Move has been made. Update the Game Board
                        TokenY = msgTokenizer.nextToken();
                        TokenZ = msgTokenizer.nextToken();
                       
                        try{
                        
                        CurValToken = msgTokenizer.nextToken();
                        new_die_val = Integer.parseInt(CurValToken);                        
                        
                        }
                        catch (NoSuchElementException e)
                        {
                            new_die_val = 0;
                        }
                        
                        move = new Move(Integer.parseInt(TokenX),Integer.parseInt(TokenY),Move.convertDir(TokenZ));
                        if (firstToken.equals("BLACK")) 
                        {
                            board.applyMove(move,Player.BLACK,new_die_val);
                            iCurrentTurn = Player.YELLOW;
                        }
                        else 
                        {
                            board.applyMove(move,Player.YELLOW,new_die_val);
                            iCurrentTurn = Player.BLACK;
                        }
                    }
                    else
                    {
                        //TimeOut or Invalid Move. Exit
                        isGameOver = true;
                        if (TokenX.equals("TIMEOUT")) System.out.println("Client "+player.getMyName()+" Timed Out");
                        else if (TokenX.equals("INVALID")) System.out.println("Client "+player.getMyName()+" performed an invalid move!");
                        else System.out.println("Unknown Error, Token = "+TokenX);
                    }
                
                }
            
            if (iMyColor == iCurrentTurn){
                move = player.chooseMove(board);
                if (player.getMyColor() == Player.BLACK) msg = new String("BLACK ");else msg = new String("YELLOW ");
                msg = msg.concat(move.toString());
                out.println(msg);
                System.out.println("Sent Move to Server: "+msg);
                if (iCurrentTurn == Player.BLACK) iCurrentTurn = Player.YELLOW;else iCurrentTurn = Player.BLACK;
            }
                
            }while (!isGameOver);
        }
        catch (Exception e) {
            System.out.println("[Connection error with server: "+szServerMachine+", or null move returned]");
            System.exit(-1);
            //System.out.println(e);
            //e.printStackTrace();
        }
    }
                
    
    public static void usage()
	{
                System.out.println("Java KnockAbout Client v1.0");
		System.out.println("Usage: GameClient <server> <port>");
                System.exit(-1);
	}
}                   
